'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ = require('lodash');
var fetch = require('isomorphic-fetch');

var _require = require('normalizr'),
    Schema = _require.Schema,
    arrayOf = _require.arrayOf,
    normalize = _require.normalize;

var _require2 = require('./response'),
    parseJSON = _require2.parseJSON,
    checkError = _require2.checkError,
    apiHandler = _require2.apiHandler,
    makeQueryString = _require2.makeQueryString;

var _require3 = require('./constants'),
    API_ROOT = _require3.API_ROOT;

/* Define schemas used for Api responses
 */


var schemaOpts = { idAttribute: '_id' };
var sessionSchema = new Schema('sessions', schemaOpts);
var recipeSchema = new Schema('recipes', schemaOpts);
var mealSchema = new Schema('meals', schemaOpts);
var userSchema = new Schema('users', schemaOpts);
var foodSchema = new Schema('foods', schemaOpts);
var ingredientFoodsSchema = new Schema('ingredientFoods', schemaOpts);
var ingredientRecipesSchema = new Schema('ingredientRecipes', schemaOpts);

var communitySchema = new Schema('communities', schemaOpts);

sessionSchema.define({
  user: userSchema
});

foodSchema.define({
  author: userSchema
});

ingredientFoodsSchema.define({
  food: foodSchema
});

recipeSchema.define({
  author: userSchema,
  ingredientFoods: arrayOf(ingredientFoodsSchema)
});

ingredientRecipesSchema.define({
  recipe: recipeSchema
});

mealSchema.define({
  author: userSchema,
  ingredientFoods: arrayOf(ingredientFoodsSchema),
  ingredientRecipes: arrayOf(ingredientRecipesSchema)
});

communitySchema.define({
  creator: userSchema
});

/* Export available schemas corresponding for every
 * Api response (used by /actions).
 */
var SCHEMAS = {
  SESSION: sessionSchema,

  USER: userSchema,
  USERS: arrayOf(userSchema),

  MEAL: mealSchema,
  MEALS: arrayOf(mealSchema),

  RECIPE: recipeSchema,
  RECIPES: arrayOf(recipeSchema),

  FOOD: foodSchema,
  FOODS: arrayOf(foodSchema),

  COMMUNITY: communitySchema,
  COMMUNITIES: arrayOf(communitySchema)
};

/* Special symbol that signals this middleware to be
 * called in the redux middleware chain (used by /actions).
 */
var CALL_API = 'Call API';

function assertValidCall(_ref) {
  var query = _ref.query,
      body = _ref.body,
      method = _ref.method,
      endpoint = _ref.endpoint,
      schema = _ref.schema,
      types = _ref.types;

  if (query && body) {
    throw new Error('Do not call with both query and body');
  }
  if (!method) {
    throw new Error('Specify a method for api call.');
  }
  if (['GET', 'POST', 'PUT', 'DELETE'].indexOf(method) === -1) {
    throw new Error('Method must be one of {GET, POST, PUT, DELETE}');
  }
  if (typeof endpoint !== 'string') {
    throw new Error('Specify a string endpoint URL.');
  }
  if (!schema) {
    throw new Error('Specify one of the exported Schemas.');
  }
  if (!Array.isArray(types) || types.length !== 4) {
    throw new Error('Expected an array of four action types.');
  }
  if (!types.every(function (type) {
    return typeof type === 'string';
  })) {
    throw new Error('Expected action types to be strings.');
  }
}

/* Middleware that interprets calls with CALL_API.
 */
var apiFn = function apiFn(store) {
  return function (next) {
    return function (action) {
      var callAPI = action[CALL_API];
      if (typeof callAPI === 'undefined') {
        return next(action);
      }

      var token = callAPI.token,
          query = callAPI.query,
          body = callAPI.body,
          method = callAPI.method,
          endpoint = callAPI.endpoint,
          schema = callAPI.schema,
          types = callAPI.types;


      assertValidCall(callAPI);

      var url = API_ROOT + '/' + endpoint;

      var _types = _slicedToArray(types, 4),
          reqType = _types[0],
          okType = _types[1],
          cancelType = _types[2],
          errType = _types[3];

      var options = {
        credentials: 'include',
        method: method,
        headers: {
          'Accept': 'application/json',
          'Content-Type': 'application/json'
        }
      };

      if (token) {
        options.headers.Authorization = token;
      }

      if (body) {
        options.body = JSON.stringify(body);
      }

      if (query && !_.isEmpty(query)) {
        url = url + '?' + makeQueryString(query);
      }

      next({ type: reqType });
      return fetch(url, options).then(parseJSON).then(apiHandler(function (json) {
        next({
          type: okType,
          response: normalize(json, schema)
        });
      }, function (error, mongoose) {
        next({ type: cancelType });
        next({
          type: errType,
          error: error,
          mongoose: mongoose
        });
      }));
    };
  };
};

module.exports = {
  CALL_API: CALL_API,
  SCHEMAS: SCHEMAS,
  apiFn: apiFn,
  parseJSON: parseJSON,
  checkError: checkError,
  apiHandler: apiHandler,
  makeQueryString: makeQueryString
};